#!/usr/bin/env python

import argparse
import random
import collections

import pyrosetta.rosetta as rosetta
import pyrosetta
#import pyrosetta.rosetta.protocols.loops.loop_mover.refine

import kic_loop_builder as kic


def parse():
    parser = argparse.ArgumentParser(description="Model loops in the given structure")
    required = parser.add_argument_group('required arguments')
    required.add_argument('-pdb',
            help="Input PDB structure (e.g. './AF2_result/B_10y_relaxed_rank_1_model_5.pdb')",
            required=True)
    required.add_argument('-loops', 
            help="Loops to be modeled (e.g. '104-120,156-162,366-380,424-433')",
            required=True)
    required.add_argument('-o', '--outfile', 
            help='Output file (e.g. output.pdb.gz)',
            required=True)
    return parser.parse_args()


def main():
    n_retries = 1
    Seg = collections.namedtuple('Seg', 's, e')
    para = parse()
    loops = []
    for s_e in para.loops.split(','):
        s, e = s_e.split('-')
        loops.append(Seg(int(s), int(e)))

    pyrosetta.init('-ex1 -ex2 -packing:use_input_sc -include_sugars -alternate_3_letter_codes pdb_sugar -out:level 3')
    scorefxn = pyrosetta.get_fa_scorefxn()
    p = pyrosetta.Pose()
    pyrosetta.pose_from_file(p, para.pdb)
    print(loops)
    random.shuffle(loops)
    print(loops)
    for lp in loops:
        poses = []
        for i in range(n_retries):
            working_p = pyrosetta.Pose()
            working_p.assign(p)
            kic.build_loop(working_p, lp.s, lp.e, kic.LoopBuildMod.Rebuild, m=5) # Productioni run
            #kic.build_loop(working_p, lp.s, lp.e, kic.LoopBuildMod.Rebuild, m=1) # Fewer inner circle iterations for faster run in test
            e = rosetta.core.pose.total_energy_from_pose(working_p)
            poses.append((e, working_p))
        poses.sort(key=lambda x:x[0])
        print(poses)
        p.assign(poses[0][1])
    p.dump_pdb(para.outfile)

if __name__ == '__main__':
    main()
